ARCHITECTURES = {
    'arm': {
        'compiler': 'aarch64-linux-gnu-gcc',
        'emulator': 'qemu-aarch64-static',
    },
    'riscv': {
        'compiler': 'riscv64-linux-gnu-gcc',
        'emulator': 'qemu-riscv64-static',
    }
}
